/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "spring.h"

Spring::Spring( QWidget *parent )
		: QGLWidget( parent )
{
	// setMouseTracking( TRUE );

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(unPasoDeformacion()) );


	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	Inicializa( );
}

void Spring::Inicializa( void )
{
	// Para la estructura general
	r.f = 0.0;
	r.k = 11.11111;
	r.m = 1.0;
	r.b = 2.66667;
	r.kint = 1.111111;

	r.dt = 0.033;
	r.theta = 0.0;

	r.dt2m = r.kint * r.dt * r.dt / r.m;

	r.k1 = 2.0 - r.dt*(r.k*r.dt + r.b)/r.m;
	r.k2 = 1.0 - r.dt*r.b/r.m;

	// Para cada componente del resorte
	r.rx.f = r.rx.k3 = 0.0;
	r.ry.f = r.ry.k3 = 0.0;

	r.rx.x_1 = r.rx.x = r.rx.x1 = 0.0;
	r.ry.x_1 = r.ry.x = r.ry.x1 = 0.0;
}

//
// This method puts the initial coordinates each time window is resized
//
void Spring::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho( -0.125, 0.125, -0.05/aspect, 0.05/aspect, -1.0, 1.0);
	else
		glOrtho(-0.125*aspect, 0.125*aspect, -0.05, 0.05, -1.0, 1.0);
}

void Spring::paintGL( void )
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */


	glLineWidth( 4.0 );
	muestraResorte(  );

	swapBuffers( );
}

void Spring::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
}

void Spring::muestraResorte(  )
{
	double px, py;

	glBegin( GL_LINES );

	glColor3f( 0.0, 0.0, 1.0);

	glVertex2d(-0.08, 0.025 ); 
	glVertex2d(-0.08,-0.025 ); 

	glColor3f( 1.0, 0.0, 0.0);
	glVertex2d(-0.08, 0.0 ); 

	px = -0.02+r.rx.x1;
	py = r.ry.x1;
	glVertex2d( px, py ); 

	glEnd(  );

	glBegin( GL_QUADS );
		glColor3f( 1.0, 0.0, 0.0);
		glVertex2d( px-0.005, py-0.005 );
		glVertex2d( px+0.005, py-0.005 );
		glVertex2d( px+0.005, py+0.005 );
		glVertex2d( px-0.005, py+0.005 );
	glEnd(  );
}

void Spring::unPasoDeformacion( void )
{
	// static int t=0;
	MRA *pr;

	pr = &r.rx;

	pr->fi = pr->x * r.dt2m;
    pr->x1 = r.k1 * pr->x - r.k2 * pr->x_1 + pr->k3 - pr->fi;
    pr->x_1 = pr->x;
    pr->x   = pr->x1;

	pr = &r.ry;

	pr->fi = pr->x * r.dt2m;
    pr->x1 = r.k1 * pr->x - r.k2 * pr->x_1 + pr->k3 - pr->fi;
    pr->x_1 = pr->x;
    pr->x   = pr->x1;

	// fprintf( stderr, "%d\n", t );
	// t++;

	updateGL();
}
	
